/*
    *
    * This file is a part of CoreGarage.
    * A setting manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/


#include <QDir>
#include <QFile>
#include <QIcon>
#include <QFileDialog>
#include <QTreeWidgetItem>
#include <QWidget>
#include <QTreeWidgetItem>
#include <QMessageBox>
#include <QFontDialog>
#include <QScroller>

#include <cprime/cprime.h>
#include <cprime/applicationdialog.h>
#include <cprime/appopenfunc.h>
#include <cprime/cplugininterface.h>
#include <cprime/themefunc.h>
#include <libarchiveqt.h>

#include "coregarage.h"
#include "ui_coregarage.h"


coregarage::coregarage(QWidget *parent)
	: QWidget(parent)
	, ui(new Ui::coregarage)
	, sm(settingsManage::initialize(true))
{
    ui->setupUi(this);

    QPalette pltt(palette());
    pltt.setColor(QPalette::Base, pltt.color(QPalette::Window));
    setPalette(pltt);

    initialize();
}

coregarage::~coregarage()
{
    delete ui;
}

void coregarage::initialize()
{
    loadSettings();
    startSetup();

    setupCoreAppsPage();
    setupCoreActionPage();
    setupOthersPage();
    setupCoreTerminalPage();
    setupCoreStuffPage();

    on_coreapps_clicked();
}

/**
 * @brief Setup ui elements
 */
void coregarage::startSetup()
{
    // all toolbuttons icon size in sideView
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(sideViewIconSize);
        }
    }

    // set stylesheet from style.qrc
    ui->sideView->setStyleSheet(CPrime::ThemeFunc::getSideViewStyleSheet());

	// In case settings file copy issue
	if (sideViewSize <= 0) {
		sideViewSize = 180;
	}
    ui->sideView->setFixedWidth(sideViewSize);

    on_autoDetect_clicked(autoDetect);

    QScroller::grabGesture(ui->scrollArea_2, QScroller::LeftMouseButtonGesture);
    QScroller::grabGesture(ui->scrollArea_3, QScroller::LeftMouseButtonGesture);
    QScroller::grabGesture(ui->scrollArea_4, QScroller::LeftMouseButtonGesture);
    QScroller::grabGesture(ui->scrollArea_5, QScroller::LeftMouseButtonGesture);

    // set starup page as coreapps page
    on_coreapps_clicked();

    ui->menu->setVisible(0);
    ui->appTitle->setAttribute(Qt::WA_TransparentForMouseEvents);
    ui->appTitle->setFocusPolicy(Qt::NoFocus);
    this->resize(800, 500);

    if ( uiMode != 0 ) {
        this->setWindowState( Qt::WindowMaximized );

        if (uiMode == 2){
            this->setWindowState(Qt::WindowMaximized);
            ui->menu->setIconSize(toolBarIconSize);
            ui->menu->setVisible(1);
            ui->menu->setChecked(false);
            ui->sideView->setVisible(0);
        }
    }


}

/**
 * @brief Loads application settings
 */
void coregarage::loadSettings()
{
    sideViewSize = sm->value("CoreApps", "SideViewSize");
    sideViewIconSize = sm->value("CoreApps", "SideViewIconSize");
    toolBarIconSize = sm->value("CoreApps", "ToolBarIconSize");
    autoDetect = sm->value("CoreApps", "AutoDetect");
    uiMode = sm->value("CoreApps", "UIMode");
}

void coregarage::setupCoreAppsPage()
{
    // general
    ui->isRecentDisable->setChecked(sm->value("CoreApps", "KeepActivities"));
    ui->setSideViewSize->setValue(sm->value("CoreApps", "SideViewSize"));
    ui->sideViewIconSize->setCurrentText(QString::number(QSize(sm->value("CoreApps", "SideViewIconSize")).width()));
    ui->listViewIconSize->setCurrentText(QString::number(QSize(sm->value("CoreApps", "ListViewIconSize")).width()));
    ui->iconViewIconSize->setCurrentText(QString::number(QSize(sm->value("CoreApps", "IconViewIconSize")).width()));
    ui->toolBarIconSize->setCurrentText(QString::number(QSize(sm->value("CoreApps", "ToolBarIconSize")).width()));
    ui->uiMode->setCurrentIndex(sm->value("CoreApps", "UIMode"));
    ui->autoDetect->setChecked(sm->value("CoreApps", "AutoDetect"));
    ui->useSystemN->setChecked(sm->value("CoreApps", "UseSystemNotification"));

    // Preferred Applications
    QString temp = sm->value("CoreApps", "Terminal");
    ui->terminals->setText(temp.count() ? temp : "Nothing");
    temp = static_cast<QString>(sm->value("CoreApps", "FileManager"));
    ui->fileManger->setText(temp.count() ? temp : "Nothing");
    temp = static_cast<QString>(sm->value("CoreApps", "ImageEditor"));
    ui->imageEditor->setText(temp.count() ? temp : "Nothing");
    temp = static_cast<QString>(sm->value("CoreApps", "MetadataViewer"));
    ui->metadataViewer->setText(temp.count() ? temp : "Nothing");
    temp = static_cast<QString>(sm->value("CoreApps", "SearchApp"));
    ui->searchApp->setText(temp.count() ? temp : "Nothing");
    temp = static_cast<QString>(sm->value("CoreApps", "RenamerApp"));
    ui->renamerApp->setText(temp.count() ? temp : "Nothing");

    // Favourite apps
    favAppsGrp = new QButtonGroup();
    favAppsGrp->addButton(ui->toolButton_2, 0);
    favAppsGrp->addButton(ui->toolButton_5, 1);
    favAppsGrp->addButton(ui->toolButton_4, 2);
    favAppsGrp->addButton(ui->toolButton_3, 3);
    favAppsGrp->addButton(ui->toolButton, 4);

    for (QAbstractButton* btn : favAppsGrp->buttons()) {
        btn->setIcon(QIcon());
        btn->setText("+");
    }

    QStringList appsList = sm->value("CoreApps", "Favourite");

    for (int i = 0; i < appsList.count(); i++) {
        CoreDesktopFile file = CoreDesktopFile(appsList.at(i));
        favAppsGrp->button(i)->setIcon(CPrime::ThemeFunc::getAppIcon(file.desktopName()));
        favAppsGrp->button(i)->setText(appsList.at(i));
    }

    connect(favAppsGrp, SIGNAL(buttonClicked(int)), this, SLOT(addRemoveApp(int)));

    // Advanced
    ui->experimentalFeatures->setChecked(sm->value("CoreApps", "EnableExperimental"));
}

void coregarage::setupCoreActionPage()
{
    m_pluginsFolders.clear();
    ui->delPluginPath->setDisabled(true);

    int count = sm->value("CoreAction", "PluginsFoldersCount");
    for (int i = 0; i < count; i++) {
        m_pluginsFolders << QString(sm->value("CoreAction", "PluginsFolders/" + QString::number(i)));
    }

    if (not m_pluginsFolders.contains(m_defaultPluginPath)) {
        m_pluginsFolders << m_defaultPluginPath;
    }

    ui->pluginPaths->clear();
    Q_FOREACH (QString pFolder, m_pluginsFolders) {
        QListWidgetItem *item = new QListWidgetItem(pFolder, ui->pluginPaths);
        item->setData(Qt::UserRole, pFolder);               // Not required
        item->setIcon(QIcon::fromTheme("document-open"));

        ui->pluginPaths->addItem(item);
    }

    // Available Widgets List
    QStringList selectedPlugins;
    selectedPlugins.clear();
    count = sm->value("CoreAction", "SelectedPluginsCount");
    for (int i = 0; i < count; i++) {
        selectedPlugins << QString(sm->value("CoreAction", "SelectedPlugins/" + QString::number(i)));
    }

    m_selectedPlugins.clear();
    ui->selectedWidgetsList->clear();
    for (int i = 0; i < count; i++) {
        QString pluginso = selectedPlugins.at(i);
        QPluginLoader loader(pluginso);
        QObject *pObject = loader.instance();

        if (pObject) {
            plugininterface *plugin = qobject_cast<plugininterface *>(pObject);

            if (!plugin) {
                continue;
            }

            QListWidgetItem *item = new QListWidgetItem(plugin->name(), ui->selectedWidgetsList);
            item->setData(Qt::UserRole, pluginso);
            m_selectedPlugins << pluginso;
            ui->selectedWidgetsList->addItem(item);
        }
    }

    ui->pluginPaths->setCurrentRow(0);

    // Load default plugins
    searchPlugin(ui->pluginPaths->currentItem());
}

void coregarage::setupCoreStuffPage()
{
    // CoreStuff
    ui->bgPath->setText(sm->value("CoreStuff", "Background"));
    ui->wallpaperPos->setCurrentIndex(sm->value("CoreStuff", "WallpaperPositon"));

    ui->keySequenceEdit->setKeySequence( QKeySequence::fromString( sm->value( "CoreSession", "DockKeyCombo" ) ) );

}

void coregarage::setupCoreTerminalPage()
{
    /* Color Schemes */
    QStringList filters = QStringList() << "*.schema" << "*.colorscheme";

    ui->colorSchemesCB->clear();
    Q_FOREACH (QString schema, QDir("/usr/share/qtermwidget5/color-schemes/").entryList(filters, QDir::Files, QDir::Name | QDir::IgnoreCase)) {
        ui->colorSchemesCB->addItem(schema.replace(".schema", "").replace(".colorscheme", ""));
    }

    ui->colorSchemesCB->setCurrentText(sm->value("CoreTerminal", "ColorScheme"));

    /* Opacity */
    ui->opacitySB->setValue(sm->value("CoreTerminal", "Opacity"));

    /* Font */
    QFont tFont = sm->value("CoreTerminal", "Font");
    ui->terminalFont->setFont(tFont);
    ui->terminalFont->setText(tFont.family() + ", " + QString::number(tFont.pointSize()));

    /* Shell program */
    ui->shellLE->setText(sm->value("CoreTerminal", "Shell"));

    ui->keytabCB->clear();
    /* Keytabs */
    QStringList filters2 = QStringList() << "*.keytab";
    Q_FOREACH (QString schema, QDir("/usr/share/qtermwidget5/kb-layouts/").entryList(filters2, QDir::Files, QDir::Name | QDir::IgnoreCase)) {
        ui->keytabCB->addItem(schema.replace(".keytab", ""));
    }

    ui->keytabCB->setCurrentText(sm->value("CoreTerminal", "KeyTab"));
    ui->cursorShapeCB->clear();
    ui->cursorShapeCB->addItems(QStringList() << "Block Cursor" << "Underline Cursor" << "IBeam Cursor");
    ui->cursorShapeCB->setCurrentIndex(sm->value("CoreTerminal", "CursorShape"));
    ui->historySB->setValue(sm->value("CoreTerminal", "HistorySize"));
}

void coregarage::setupOthersPage()
{
    // corefm
    ui->checkThumbs->setChecked(sm->value("CoreFM", "ShowThumb"));
    ui->startPath->setText(sm->value("CoreFM", "StartupPath"));
    ui->view->setCurrentIndex(sm->value("CoreFM", "ViewMode"));

    // corepad
    QFont txFont = sm->value("CorePad", "Font");
    ui->textViewFont->setFont(txFont);
    ui->textViewFont->setText(txFont.family() + ", " + QString::number(txFont.pointSize()));

    // coreshot
    ui->ssLocation->setText(sm->value("CoreShot", "SaveLocation"));
    ui->afterShootTaken->setCurrentIndex(sm->value("CoreShot", "AfterShootTaken"));

    // corepdf
    ui->pdfPageView->setCurrentIndex(sm->value("CorePDF", "ZoomMode"));
    ui->pdfPageLayout->setCurrentIndex(sm->value("CorePDF", "PageMode"));

    // corekeyboard
    ui->keyboardLayout->setCurrentIndex(sm->value("CoreKeyboard", "Layout"));
    ui->showAutoSuggestions->setChecked(sm->value("CoreKeyboard", "AutoSuggest"));
    ui->ckFont->setCurrentFont(sm->value("CoreKeyboard", "Font"));
}

void coregarage::saveSettings()
{
    // coreapps
    bool sr(ui->isRecentDisable->isChecked());
    sm->setValue("CoreApps", "KeepActivities", sr);
    bool sn(ui->useSystemN->isChecked());
    sm->setValue("CoreApps", "UseSystemNotification", sn);

    if (!sr) {
        QFile f(CPrime::Variables::CC_CoreApps_RecentActFilePath());
        f.remove();
    }

    sm->setValue("CoreApps", "SideViewSize", ui->setSideViewSize->value());
    sm->setValue("CoreApps", "UIMode", ui->uiMode->currentIndex());
	sm->setValue("CoreApps", "AutoDetect", ui->autoDetect->isChecked());
    sm->setValue("CoreApps", "SideViewIconSize", QSize(ui->sideViewIconSize->currentText().toInt(), ui->sideViewIconSize->currentText().toInt()));
    sm->setValue("CoreApps", "IconViewIconSize", QSize(ui->iconViewIconSize->currentText().toInt(), ui->iconViewIconSize->currentText().toInt()));
    sm->setValue("CoreApps", "ListViewIconSize", QSize(ui->listViewIconSize->currentText().toInt(), ui->listViewIconSize->currentText().toInt()));
    sm->setValue("CoreApps", "ToolBarIconSize", QSize(ui->toolBarIconSize->currentText().toInt(), ui->toolBarIconSize->currentText().toInt()));
    sm->setValue("CoreApps", "Terminal", ui->terminals->text());
    sm->setValue("CoreApps", "FileManager", ui->fileManger->text());
    sm->setValue("CoreApps", "ImageEditor", ui->imageEditor->text());
    sm->setValue("CoreApps", "MetadataViewer", ui->metadataViewer->text());
    sm->setValue("CoreApps", "SearchApp", ui->searchApp->text());
    sm->setValue("CoreApps", "RenamerApp", ui->renamerApp->text());

    QStringList favApps;
    for (int i = 0; i < 5; i++) {
        QString btnTxt = favAppsGrp->button(i)->text().replace("&", "");

        if ((btnTxt != "...") and (btnTxt != "+")) {
            favApps << btnTxt;
        }
    }

    sm->setValue("CoreApps", "Favourite", favApps);
    sm->setValue("CoreApps", "EnableExperimental", ui->experimentalFeatures->isChecked());

    // corefm
    sm->setValue("CoreFM", "StartupPath", ui->startPath->text());
    sm->setValue("CoreFM", "ShowThumb", ui->checkThumbs->isChecked());
    sm->setValue("CoreFM", "ViewMode", ui->view->currentIndex() /*? true : false*/);

    // corepdf
    sm->setValue("CorePDF", "ZoomMode", ui->pdfPageView->currentIndex());
    sm->setValue("CorePDF", "PageMode", ui->pdfPageLayout->currentIndex());

    // corescreenshot
    sm->setValue("CoreShot", "SaveLocation", ui->ssLocation->text());
    sm->setValue("CoreShot", "AfterShootTaken", ui->afterShootTaken->currentIndex());

    // coreaction
    for (int i = 0; i < ui->selectedWidgetsList->count(); i++) {
        sm->setValue("CoreAction", "SelectedPlugins/" + QString::number(i), ui->selectedWidgetsList->item(i)->data(Qt::UserRole).toString());
    }
    sm->setValue("CoreAction", "SelectedPluginsCount", ui->selectedWidgetsList->count());

    for (int i = 0; i < ui->pluginPaths->count(); i++) {
        sm->setValue("CoreAction", "PluginsFolders/" + QString::number(i), ui->pluginPaths->item(i)->text());
    }
    sm->setValue("CoreAction", "PluginsFoldersCount", ui->pluginPaths->count());

    // corepad
    sm->setValue("CorePad", "Font", ui->textViewFont->font());

    // coreterminal
    sm->setValue("CoreTerminal", "ColorScheme", ui->colorSchemesCB->currentText());
    sm->setValue("CoreTerminal", "KeyTab", ui->keytabCB->currentText());
    sm->setValue("CoreTerminal", "Opacity", ui->opacitySB->value());
    sm->setValue("CoreTerminal", "Font", ui->terminalFont->font());
    sm->setValue("CoreTerminal", "Shell", ui->shellLE->text());
    sm->setValue("CoreTerminal", "CursorShape", ui->cursorShapeCB->currentIndex());
    sm->setValue("CoreTerminal", "HistorySize", ui->historySB->value());

    // corekeyboard
    sm->setValue("CoreKeyboard", "Layout", ui->keyboardLayout->currentIndex());
    sm->setValue("CoreKeyboard", "AutoSuggest", ui->showAutoSuggestions->isChecked());
    sm->setValue("CoreKeyboard", "Font", ui->ckFont->currentText());

    // corestuff
    sm->setValue("CoreStuff", "Background", ui->bgPath->text());
    sm->setValue("CoreStuff", "WallpaperPositon", ui->wallpaperPos->currentIndex());

    // inform the user
    CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Settings Applied", "All CoreApps needs to restart to see the effect", this);
}

void coregarage::on_cancel_clicked()
{
    close();
}

void coregarage::on_ok_clicked()
{
    ui->cancel->setText("Close");

    saveSettings();
}

void coregarage::on_coreapps_clicked()
{
    pageClick(ui->coreapps, 0, tr("CoreApps"));
}

void coregarage::on_coreaction_clicked()
{
    pageClick(ui->coreaction, 1, tr("CoreAction"));
}

void coregarage::on_coreterminal_clicked()
{
    pageClick(ui->coreterminal, 2, tr("CoreTerminal"));
}

void coregarage::on_others_clicked()
{
    pageClick(ui->others, 3, "Others");
}

void coregarage::pageClick(QToolButton *btn, int i, QString title)
{
    // first do this to get all the space for contents
    if (uiMode == 2){
        ui->sideView->setVisible(0);
    }

    // all button checked false
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        b->setChecked(false);
    }

    btn->setChecked(true);
    ui->selectedsection->setText(title);
    ui->pages->setCurrentIndex(i);
    this->setWindowTitle(title + " - CoreGarage");
}

// ---------------------- CoreAction page functionality ----------------

void coregarage::searchPlugin(QListWidgetItem *item)
{
    if (!item) {
        return;
    }

    ui->availableWidgitsList->clear();

    QString pFolder = item->data(Qt::UserRole).toString();
    QDir pluginDir(pFolder);

    Q_FOREACH (QString pluginSo, pluginDir.entryList(QStringList() << "*.so", QDir::Files)) {

        QPluginLoader loader(pluginDir.absoluteFilePath(pluginSo));
        QObject *pObject = loader.instance();

        if (pObject) {
            plugininterface *plugin = qobject_cast<plugininterface *>(pObject);

            if (plugin and not m_selectedPlugins.contains(pluginDir.absoluteFilePath(pluginSo))) {
                QListWidgetItem *item = new QListWidgetItem(plugin->name(), ui->availableWidgitsList);
                item->setData(Qt::UserRole, pluginDir.absoluteFilePath(pluginSo));
                ui->availableWidgitsList->addItem(item);
            }
        }
    }
}

void coregarage::on_addPluginPath_clicked()
{
    const QString p = QFileDialog::getExistingDirectory(this, "Select plugin location");

    if (p.isNull() || p.isEmpty()) {
        return;
    }

    m_pluginsFolders << p;

    QIcon q(QIcon::fromTheme("document-open"));
    QListWidgetItem *item = new QListWidgetItem();
    item->setText(p);
    item->setIcon(q);
    item->setData(Qt::UserRole, p);
    ui->pluginPaths->addItem(item);

    searchPlugin(ui->pluginPaths->currentItem());
}

void coregarage::on_delPluginPath_clicked()
{
    if (ui->pluginPaths->currentItem()) {
        QListWidgetItem *it = ui->pluginPaths->takeItem(ui->pluginPaths->currentIndex().row());
        delete it;

        searchPlugin(ui->pluginPaths->currentItem());
    }
}

void coregarage::on_addPlugin_clicked()
{
    QListWidgetItem *item = ui->availableWidgitsList->currentItem();

    if (item) {
        QListWidgetItem *temp = item->clone();
        ui->selectedWidgetsList->addItem(temp);
        delete ui->availableWidgitsList->takeItem(ui->availableWidgitsList->currentRow());
    }
}

void coregarage::on_removePlugin_clicked()
{
    QListWidgetItem *item = ui->selectedWidgetsList->currentItem();

    if (item) {
        QListWidgetItem *temp = item->clone();
        ui->availableWidgitsList->addItem(temp);
        ui->selectedWidgetsList->takeItem(ui->selectedWidgetsList->currentRow());
        ui->availableWidgitsList->sortItems();
    }
}

void coregarage::on_movePluginUp_clicked()
{
    QListWidgetItem *curr = ui->selectedWidgetsList->currentItem();

    if (!curr) {
        return;
    }

    int currIndex = ui->selectedWidgetsList->currentRow();

    if (currIndex == 0) {
        return;
    }

    int prevIndex = currIndex - 1;

    curr = ui->selectedWidgetsList->takeItem(currIndex);
    ui->selectedWidgetsList->insertItem(prevIndex, curr);
    ui->selectedWidgetsList->setCurrentItem(curr);
}

void coregarage::on_movePluginDown_clicked()
{
    QListWidgetItem *curr = ui->selectedWidgetsList->currentItem();

    if (!curr) {
        return;
    }

    int currIndex = ui->selectedWidgetsList->currentRow();

    if (currIndex == ui->selectedWidgetsList->count() - 1) {
        return;
    }

    int nextIndex = currIndex + 1;

    curr = ui->selectedWidgetsList->takeItem(currIndex);
    ui->selectedWidgetsList->insertItem(nextIndex, curr);
    ui->selectedWidgetsList->setCurrentItem(curr);
}

void coregarage::on_pluginPaths_itemSelectionChanged()
{
    QListWidgetItem *item = ui->pluginPaths->currentItem();

    if (not item) {
        ui->delPluginPath->setDisabled(true);
    } else if (item->text().startsWith("/usr/lib/coreapps/plugins")) {
        ui->delPluginPath->setDisabled(true);
    } else {
        ui->delPluginPath->setEnabled(true);
    }


    QStringList selectedPlugins = m_selectedPlugins;
    ui->selectedWidgetsList->clear();

    for (int i = 0; i < selectedPlugins.count(); i++) {
        QString pluginso = selectedPlugins.at(i);
        QPluginLoader loader(pluginso);
        QObject *pObject = loader.instance();

        if (pObject) {
            plugininterface *plugin = qobject_cast<plugininterface *>(pObject);

            if (!plugin) {
                continue;
            }

            QListWidgetItem *item = new QListWidgetItem(plugin->name(), ui->selectedWidgetsList);
            item->setData(Qt::UserRole, pluginso);
            ui->selectedWidgetsList->addItem(item);
        }
    }

    searchPlugin(item);
}

void coregarage::on_availableWidgitsList_itemSelectionChanged()
{
    if (ui->availableWidgitsList->currentItem()) {
        ui->addPlugin->setEnabled(true);
    } else {
        ui->addPlugin->setEnabled(false);
    }
}

void coregarage::on_selectedWidgetsList_itemSelectionChanged()
{
    if (ui->selectedWidgetsList->currentItem()) {
        ui->removePlugin->setEnabled(true);
    } else {
        ui->removePlugin->setEnabled(false);
    }
}

// ---------------------------------------------------------------------

void coregarage::on_browseF_clicked()
{
    ui->startPath->setText(QFileDialog::getExistingDirectory(this, "Select Startup Path for CoreFM"));
}

void coregarage::on_browseS_clicked()
{
    ui->ssLocation->setText(QFileDialog::getExistingDirectory(this, "Select Save Location for Screenshot"));
}

void coregarage::on_backUp_clicked()
{
    const QString saveFileName = QFileDialog::getSaveFileName (
        this,
        "CoreGarage | Backup",
        QDir::home().filePath( "CoreApps_Backup.zip" ),
        "Archives (*.zip *.tar.xz);;All Files (*.*)"
    );

    if ( saveFileName.isEmpty() )
        return;

    const QString configsDir = CPrime::Variables::CC_CoreApps_ConfigDir();

    LibArchiveQt *arc = new LibArchiveQt( saveFileName );
    QObject::connect(arc, &LibArchiveQt::jobComplete, this, &coregarage::showBackupMessage);

    /* New mode in libarchive: specially for saving configuration settings. */
    arc->updateInputFiles( { configsDir }, LibArchiveQt::RelativeToHome );
    arc->createArchive();
}

void coregarage::on_restore_clicked()
{
    // select the backup file
    const QString path = QFileDialog::getOpenFileName(this, "Select the backup file");

    // Extract the folder in .config folder
    if (path.count()) {
        if (QString(path.split('/').last()) == "CoreApps_Backup.zip") {
            int files = 0;
            QDirIterator it(CPrime::Variables::CC_CoreApps_ConfigDir(), QDir::AllEntries | QDir::System | QDir::NoDotAndDotDot | QDir::NoSymLinks);

            while (it.hasNext()) {
                it.next();

                if (it.fileInfo().isFile()) {
                    files++;
                }
            }

            if (files) {
                QString msg = QString("There are old settings file\nDo you want to delete them?");
                QMessageBox message(QMessageBox::Question, "File Exists", msg, QMessageBox::Yes | QMessageBox::No, this);
                message.setWindowIcon(QIcon::fromTheme("coregarage"));

                int reply = message.exec();
                if (reply == QMessageBox::No) {
                    CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Settings not restored.", "User canceled the task.", this);
                    return;
                } else {
                    QDir ds(QDir::homePath() + "/.config/coreapps");

                    if (!ds.removeRecursively()) {
                        CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Settings not restored.", "Something bad happned.", this);
                        return;
                    }
                }
            }

            LibArchiveQt *arc = new LibArchiveQt(path);
			QObject::connect(arc, &LibArchiveQt::jobComplete, this, &coregarage::showRestoreMessage);
            arc->setDestination( QDir::homePath() );
            arc->extractArchive();

        } else {
            // Function from utilities.cpp
            CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Wrong file selected!!!", "Please check the file name with correct formate", this);
        }
    } else {
        CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Nothing selected!!!", "Select valid file.", this);
    }
}

void coregarage::showBackupMessage()
{
    CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Backup successfull", "Settings backup successfully done in selected folder.", this);
}

void coregarage::showRestoreMessage()
{
    CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Restore successfull", "Settings restore successfully, Please restart the app.", this);
}

void coregarage::on_setTerminals_clicked()
{
    // Select application in the dialog
    ApplicationDialog *dialog = new ApplicationDialog(this);

    if (dialog->exec()) {
        if (dialog->getCurrentLauncher().compare("") != 0) {
            ui->terminals->setText(dialog->getCurrentLauncher());
        }
    }
}

void coregarage::on_setFileManger_clicked()
{
    // Select application in the dialog
    ApplicationDialog *dialog = new ApplicationDialog(this);

    if (dialog->exec()) {
        if (dialog->getCurrentLauncher().compare("") != 0) {
            ui->fileManger->setText(dialog->getCurrentLauncher());
        }
    }
}

void coregarage::on_setImageEditor_clicked()
{
    // Select application in the dialog
    ApplicationDialog *dialog = new ApplicationDialog(this);

    if (dialog->exec()) {
        if (dialog->getCurrentLauncher().compare("") != 0) {
            ui->imageEditor->setText(dialog->getCurrentLauncher());
        }
    }
}

void coregarage::on_setMetadataViewer_clicked()
{
    // Select application in the dialog
    ApplicationDialog *dialog = new ApplicationDialog(this);

    if (dialog->exec()) {
        if (dialog->getCurrentLauncher().compare("") != 0) {
            ui->metadataViewer->setText(dialog->getCurrentLauncher());
        }
    }
}

void coregarage::on_setSearchApp_clicked()
{
    // Select application in the dialog
    ApplicationDialog *dialog = new ApplicationDialog(this);

    if (dialog->exec()) {
        if (dialog->getCurrentLauncher().compare("") != 0) {
            ui->searchApp->setText(dialog->getCurrentLauncher());
        }
    }
}

void coregarage::on_setRenamerApp_clicked()
{
    // Select application in the dialog
    ApplicationDialog *dialog = new ApplicationDialog(this);

    if (dialog->exec()) {
        if (dialog->getCurrentLauncher().compare("") != 0) {
            ui->renamerApp->setText(dialog->getCurrentLauncher());
        }
    }
}

void coregarage::addRemoveApp(int i)
{
    QToolButton *appBtn = qobject_cast<QToolButton *>(favAppsGrp->button(i));

    /* If button has text, then add app */
    if (appBtn->text() == "+") {
        ApplicationDialog *dialog = new ApplicationDialog(this);
//        dialog->getApplications();

        if (dialog->exec()) {
            QString curr = dialog->getCurrentLauncher();

            if (not curr.isEmpty()) {
                CoreDesktopFile file = CoreDesktopFile(curr);
                appBtn->setText(curr);
                appBtn->setIcon(CPrime::ThemeFunc::getAppIcon(file.desktopName()));
                appBtn->setToolButtonStyle(Qt::ToolButtonIconOnly);
            }
        }
    } else { /* Button has some text */
        appBtn->setIcon(QIcon());
        appBtn->setText("+");
    }
}

void coregarage::showSideView()
{
    if (ui->sideView->isVisible())
        ui->sideView->setVisible(0);
    else
        ui->sideView->setVisible(1);
}

void coregarage::on_textViewFont_clicked()
{
    bool ok;
    QFont tFont = QFontDialog::getFont(&ok, sm->value("CorePad", "Font"), this, "Choose TextView Font", QFontDialog::MonospacedFonts);

    if (ok) {
        ui->textViewFont->setFont(tFont);
        ui->textViewFont->setText(tFont.family() + ", " + QString::number(tFont.pointSize()));
    }
}

void coregarage::on_terminalFont_clicked()
{
    bool ok;
    QFont tFont = QFontDialog::getFont(&ok, sm->value("CoreTerminal", "Font"), this, "Choose Terminal Font", QFontDialog::MonospacedFonts);

    if (ok) {
        ui->terminalFont->setFont(tFont);
        ui->terminalFont->setText(tFont.family() + ", " + QString::number(tFont.pointSize()));
    }
}

void coregarage::on_setBG_clicked()
{
    ui->bgPath->setText(QFileDialog::getOpenFileName(this, "Select Baackground Path for CoreStuff", QDir::homePath()));
}

void coregarage::on_restoreDefault_clicked()
{
    QString msg = QString("All settings will be replaced by default settings.\nDo you want to continue?");
    QMessageBox message(QMessageBox::Question, "Restore Default", msg, QMessageBox::Yes | QMessageBox::No, this);
    message.setWindowIcon(QIcon::fromTheme("coregarage"));

    int reply = message.exec();
    if (reply == QMessageBox::No) {
        return;
    }

    QString defaultSett = CPrime::Variables::CC_CoreApps_Default_ConfigFilePath();
    QString userSett = CPrime::Variables::CC_CoreApps_ConfigFilePath();

    bool done = true;

    // Copy default settings file to user settings file
    QString stext;
    QFile dFile(defaultSett);
    done &= dFile.open(QIODevice::ReadOnly);
    QTextStream in(&dFile);
    stext = in.readAll();
    dFile.close();

    QFile uFile(userSett);
    done &= uFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
    QTextStream out(&uFile);
    out << stext;
    uFile.close();

//    bool done = QFile(userSett).remove();

    if (!done) {
        CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Cannot restore default settings.", "Somthing bad happned.", this);
        return;
    }

    sm->setUserDefultSettings();

    QTimer::singleShot(1000, this, &coregarage::initialize);
    CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Restore successful", "Settings restore successfully to defaults.", this);
}

void coregarage::on_help_clicked()
{
    QString msg = QString("All Settings, Activites, Pins, Alarms, Reminders \nWill be backed up or restored");
    QMessageBox message(QMessageBox::Information, "Backup and Restore", msg, QMessageBox::Ok, this);
    message.setWindowIcon(QIcon::fromTheme("coregarage"));
    message.exec();
}


void coregarage::on_testNotif_clicked()
{
    if(ui->useSystemN->isChecked())
        CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Using System", "Notification Working.", this);
    else
        CPrime::InfoFunc::messageEngine("CoreGarage", "coregarage", "Using C Suite Internal", "Notification Working.", this);
}

void coregarage::on_appTitle_clicked()
{
    showSideView();
}

void coregarage::on_menu_clicked(bool checked)
{
    Q_UNUSED(checked)
    showSideView();
}

void coregarage::on_autoDetect_clicked(bool checked)
{
    if(checked){
        ui->uiMode->setEnabled(false);
        ui->autoDetect->setChecked(true);
    }
    else {
        ui->uiMode->setEnabled(true);
        ui->autoDetect->setChecked(false);
    }
}
